@@ -3,6 +3,7 @@ import datetime
 import re
 import types
 from abc import ABC
+from collections.abc import Mapping
 from inspect import isfunction, isclass, getdoc, signature
 import warnings
 
@@ -20,7 +21,7 @@ COMMENT_STR = ';;'
 SEP_INP = COMMENT_STR + "_" * 100
 COMMENT_EMPTY_SECTION = COMMENT_STR + ' No Data'
 
-_TYPES_NO_COPY = (int, float, str, datetime.date, datetime.time)
+_TYPES_NO_COPY = (type(None), int, float, str, datetime.date, datetime.time)
 
 
 class SwmmInputWarning(UserWarning): ...
@@ -39,6 +40,7 @@ class CustomDict:
 
     Imitates :class:`collections.UserDict` (:term:`dict-like <mapping>`), but operations only effect ``self._data``.
     """
+
     def __init__(self, d=None, **kwargs):
         if d is None:
             self._data = kwargs
@@ -111,6 +113,10 @@ class CustomDict:
     # def id(self):
     #     return id(self)
 
+
+Mapping.register(CustomDict)
+
+
 class InpSectionABC(ABC, CustomDict):
     """
     Abstract class for ``.inp``-file sections without objects.
@@ -203,6 +209,7 @@ class InpSectionGeneric(InpSectionABC, ABC):
     Attributes:
         _label (str): label of the section
     """
+
     def __init__(self, *args, **kwargs):
         InpSectionABC.__init__(self, *args, **kwargs)
 
@@ -246,13 +253,16 @@ class InpSection(InpSectionABC):
         create an object for ``.inp``-file sections with objects (i.e. nodes, links, subcatchments, raingages, ...)
 
         Args:
-            section_object (type[BaseSectionObject]): object class which is stored in this section.
+            section_object (BaseSectionObject | similar): object class which is stored in this section.
                 This information is used to set the index of the section and
                 to decide if the section can be exported (converted to a string) as a table.
         """
         super().__init__()
         self._section_object = section_object
 
+        from .section_types import SECTIONS_MULTI_TYPES
+        self._possible_object_types = SECTIONS_MULTI_TYPES.get(section_object._section_label, (section_object,))
+
     # def __repr__(self):
     #     # return CustomDict.__repr__(self)
     #     return str(self)
@@ -272,7 +282,7 @@ class InpSection(InpSectionABC):
                               f'Needed keys: "{self._identifier}"\n'
                               f'Given keys: "{key}"', SwmmInputWarning)
 
-        if not isinstance(value, self._section_object):
+        if not isinstance(value, self._possible_object_types):
             warnings.warn(f'Wrong section-object type (Unknown Behaviour)\n'
                           f'Needed type: "{self._section_object}"\n'
                           f'Given type: "{type(value)}"', SwmmInputWarning)
@@ -333,8 +343,8 @@ class InpSection(InpSectionABC):
         for obj in items:
             try:
                 self.add_obj(obj)
-            except:
-                print()
+            except Exception as e:
+                print(e, 'error adding objects to section')
 
     def add_obj(self, obj):
         """
@@ -343,7 +353,7 @@ class InpSection(InpSectionABC):
         Args:
             obj (BaseSectionObject): new object
         """
-        self[obj.get(self._identifier)] = obj
+        self[obj.section_key] = obj
 
     def add_inp_lines(self, multi_line_args):
         """
@@ -599,16 +609,8 @@ class InpSectionGeo(InpSection):
         return GeoSeries({label: item.geo for label, item in self.items()}, crs=crs, name='geometry')
 
 
-def split_line_with_quotes(line):
-    if isinstance(line, (list, tuple)):
-        line = ' '.join(line)
-    if '"' not in line:
-        return line.split()
-    return re.findall(r'("[^"]*"|[^" ]+)', line)
-
-
 def free_attributes(key):
-    # TODO maybe laster?!
+    # TODO maybe later?!
     return key  # .lower()  # .replace('_', '') -> .casefold() betten than fold
 
 
@@ -623,7 +625,7 @@ class BaseSectionObject(ABC):
     _identifier = ''
     """str: attribute of an object which will be used as identifiers"""
     _table_inp_export = True
-    """bool: if an section is writeable as table. Default ist True"""
+    """bool: if a section is writeable as table. Default ist True"""
     _section_class = InpSection
     """class: section class to identify functionality"""
     _section_label = ''
@@ -732,6 +734,10 @@ class BaseSectionObject(ABC):
     # def id(self):
     #     return id(self)
 
+    @property
+    def section_key(self):  # get key of object for section
+        return self.get(self._identifier)
+
     def _to_debug_string(self):
         """for debugging purposes
 
@@ -745,7 +751,7 @@ class BaseSectionObject(ABC):
 
     @property
     def _short_debug_string(self):
-        return f'{self.__class__.__name__}({self[self._identifier]})'
+        return f'{self.__class__.__name__}({self.section_key})'
 
     def to_inp_line(self):
         """
@@ -789,6 +795,22 @@ class BaseSectionObject(ABC):
         except TypeError as e:
             raise SwmmInputWarning(f'{e} | {self}')
 
+    @classmethod
+    def _prepare_convert_lines(cls, lines):
+        # print(f'BYTES: {len(lines):>9_d}', )
+        if isinstance(lines, str):
+            if len(lines) > 10_000_000:
+                n_lines = lines.strip().count('\n') + 1
+                # to create a progressbar in the reading process
+                # only needed with big (> 200 MB) files
+                return tqdm(txt_to_lines(lines), desc=cls.__name__, total=n_lines, postfix='Read')
+            else:
+                return txt_to_lines(lines)
+        elif isinstance(lines, pd.DataFrame):
+            return lines.values
+        else:
+            return lines
+
     @classmethod
     def create_section(cls, lines=None):
         """
@@ -803,26 +825,14 @@ class BaseSectionObject(ABC):
             InpSection: new section of this object type
         """
         sec = cls._section_class(cls)
-        # import sys
-        if lines is not None:
-            # print(f'BYTES: {len(lines):>9_d}', )
-            if isinstance(lines, str):
-                if len(lines) > 10_000_000:
-                    n_lines = lines.strip().count('\n') + 1
-                    # to create a progressbar in the reading process
-                    # only needed with big (> 200 MB) files
-                    lines_iter = tqdm(txt_to_lines(lines), desc=cls.__name__, total=n_lines, postfix='Read')
-                else:
-                    lines_iter = txt_to_lines(lines)
-            elif isinstance(lines, pd.DataFrame):
-                lines_iter = lines.values
-            else:
-                lines_iter = lines
+        if lines is None:
+            return sec
 
-            sec.add_inp_lines(lines_iter)
+        lines_iter = cls._prepare_convert_lines(lines)
+        sec.add_inp_lines(lines_iter)
 
-            if isinstance(lines_iter, tqdm):
-                lines_iter.close()
+        if isinstance(lines_iter, tqdm):
+            lines_iter.close()
 
         return sec
 
@@ -898,12 +908,20 @@ def dataframe_to_inp_string(df, index=True):
     if not index:
         c.columns = [COMMENT_STR + c.columns[0]] + list(c.columns)[1:]
 
-    return c.applymap(type2str).to_string(sparsify=False,
-                                          line_width=999999,
-                                          max_rows=999999,
-                                          max_cols=999999,
-                                          max_colwidth=999999,
-                                          index=index)
+    if pd.__version__.startswith('2.1.'):
+        return c.map(type2str).to_string(sparsify=False,
+                                         line_width=999999,
+                                         max_rows=999999,
+                                         max_cols=999999,
+                                         max_colwidth=999999,
+                                         index=index)
+    else:
+        return c.applymap(type2str).to_string(sparsify=False,
+                                              line_width=999999,
+                                              max_rows=999999,
+                                              max_cols=999999,
+                                              max_colwidth=999999,
+                                              index=index)
 
 
 ########################################################################################################################
@@ -1043,7 +1061,7 @@ def check_order(inp, order_list=None):
     if order_list is None:
         order_list = SECTION_ORDER_DEFAULT
     order = [order_list.index(o) if o in order_list else len(order_list) for o in inp]
-    return all((order[i+1] - order[i]) > 0 for i in range(len(order)-1))
+    return all((order[i + 1] - order[i]) > 0 for i in range(len(order) - 1))
 
 
 def _sort_by(key, sections_order):
@@ -1053,7 +1071,7 @@ def _sort_by(key, sections_order):
         return len(sections_order)
 
 
-re_int = re.compile('(\d+)')
+re_int = re.compile(r'(\d+)')
 
 
 def natural_keys(text):
@@ -1107,7 +1125,7 @@ def section_to_string(section, fast=True, sort_objects_alphabetical=False):
         return ''
 
     else:
-        print(section, '?'*10)
+        print(section, '?' * 10)
 
 
 def iter_section_lines(section, sort_objects_alphabetical=False):
