@@ -9,7 +9,7 @@ from .edit import (combine_conduits, combine_conduits_keep_slope, combine_vertic
                    delete_subcatchment, dissolve_conduit, flip_link_direction, move_flows, rename_link, rename_node,
                    rename_subcatchment, rename_timeseries, split_conduit, remove_quality_model, delete_pollutant)
 from .filter import (filter_nodes, filter_links_within_nodes, filter_links, filter_subcatchments,
-                     create_sub_inp, )
+                     create_sub_inp, filter_tags)
 from .geo import (transform_coordinates, complete_vertices, reduce_vertices, complete_link_vertices,
                   simplify_link_vertices, simplify_vertices, )
 
@@ -28,7 +28,7 @@ except ImportError as e:
 
 from .graph import (inp_to_graph, get_path, get_path_subgraph, next_links, next_links_labels, next_nodes,
                     previous_links, previous_links_labels, previous_nodes, links_connected, number_in_out,
-                    downstream_nodes, upstream_nodes, get_network_forks, split_network, conduit_iter_over_inp,
+                    get_downstream_nodes, get_upstream_nodes, get_network_forks, split_network, conduit_iter_over_inp,
                     subcatchments_connected)
 from .macros import (find_node, find_link, calc_slope, conduit_slopes, conduits_are_equal, update_no_duplicates,
                      increase_max_node_depth, set_times, combined_subcatchment_frame, delete_sections,
@@ -37,7 +37,8 @@ from .plotting_longitudinal import plot_longitudinal, animated_plot_longitudinal
 from .plotting_map import (plot_map, init_empty_map_plot, add_node_map, add_link_map, add_subcatchment_map,
                            add_node_labels, set_inp_dimensions, add_backdrop)
 from .reduce_unneeded import (reduce_curves, reduce_controls, simplify_curves, reduce_raingages,
-                              remove_empty_sections, reduce_timeseries, reduce_pattern)
+                              remove_empty_sections, reduce_timeseries, reduce_pattern, reduce_snowpacks,
+                              reduce_hydrographs, reduce_report_objects)
 from .split_inp_file import split_inp_to_files, read_split_inp_file
 from .summarize import print_summary
 from ._helpers import get_used_curves, print_warning
