@@ -1,3 +1,5 @@
+import os.path
+
 from tqdm.auto import tqdm
 
 from .collection import nodes_dict, links_dict
@@ -8,6 +10,16 @@ from .._type_converter import is_equal
 from ..helpers import BaseSectionObject, _sort_by
 
 
+def reverse_dict(original_dict):
+    new_dict = {}
+    for key, value in original_dict.items():
+        if value not in new_dict:
+            new_dict[value] = [key]
+        else:
+            new_dict[value].append(key)
+    return new_dict
+
+
 class CompareSections:
     def __init__(self, section_1, section_2, precision=3):
         """
@@ -36,7 +48,7 @@ class CompareSections:
                         self.dict_not_equal[key] = f'{section_1[key]} != {section_2[key]}'
                     else:
                         diff = []
-                        if type(section_1[key]) != type(section_2[key]):
+                        if type(section_1[key]) is not type(section_2[key]):
                             diff.append(f'{type(section_1[key]).__name__} != {type(section_2[key]).__name__}')
                         else:
                             for param in section_1[key].to_dict_():
@@ -65,16 +77,24 @@ class CompareSections:
         """
         s_warnings = ''
         if self.dict_not_equal:
-            s_warnings += (f'not equal ({len(self.dict_not_equal)}): \n    '
-                           + '\n    '.join([f'{k}: {v}' for k, v in self.dict_not_equal.items()]) + '\n')
+            unique_changes = set(self.dict_not_equal.values())
+            s_warnings += f'not equal ({len(self.dict_not_equal)}): \n    '
+
+            if len(unique_changes) < len(self.dict_not_equal)*0.5:
+                dict_not_equal_r = reverse_dict(self.dict_not_equal)
+                s_warnings += '\n    '.join([f'{k}: (n={len(v)}){v}' for k, v in dict_not_equal_r.items()]) + '\n'
+            else:
+                s_warnings += '\n    '.join([f'{k}: {v}' for k, v in self.dict_not_equal.items()]) + '\n'
 
         if self.set_labels_not_in_1:
             s_warnings += (f'not in inp1 ({len(self.set_labels_not_in_1)}): '
                            + ' | '.join(map(str, sorted(self.set_labels_not_in_1))) + '\n')
+            # TODO common pattern
 
         if self.set_labels_not_in_2:
             s_warnings += (f'not in inp2 ({len(self.set_labels_not_in_2)}): '
                            + ' | '.join(map(str, sorted(self.set_labels_not_in_2))) + '\n')
+            # TODO common pattern
 
         return (s_warnings or 'good!\n') + f'{len(self.set_labels_equal)}/{self.len_full} objects are equal\n'
 
@@ -122,7 +142,7 @@ def compare_sections(sec1, sec2, precision=3):
     return CompareSections(sec1, sec2, precision=precision).get_diff_string()
 
 
-def compare_inp_files(fn1, fn2, precision=2, skip_section=None, sep='\n' + '#' * 100):
+def compare_inp_files(fn1, fn2, precision=2, skip_section=None, sep='\n' + '#' * 100, show_progressbar=True):
     """
     compare two inp files and get the differences as string output
 
@@ -131,21 +151,23 @@ def compare_inp_files(fn1, fn2, precision=2, skip_section=None, sep='\n' + '#' *
         fn2 (str): filename for the second inp file
         precision (int): number of relevant decimal places
         skip_section (list): skip sections if you don't care for specific changes
+        show_progressbar (bool): show progress bar
 
     Returns:
         str: differences of the files
     """
-    s = (f'Comparing \n'
-         f'   "{fn1}" (=inp1)\n'
+    parent = os.path.commonpath([str(fn1), str(fn2)])
+    s = (f'Comparing ({parent})\n'
+         f'   "{str(fn1).replace(parent, "")}" (=inp1)\n'
          f'   to\n'
-         f'   "{fn2}" (=inp2)\n'
+         f'   "{str(fn2).replace(parent, "")}" (=inp2)\n'
          f'Ignoring Sections: {skip_section}\n')
     inp1 = SwmmInput.read_file(fn1)
     inp2 = SwmmInput.read_file(fn2)
-    return s + compare_inp_objects(inp1, inp2, precision=precision, skip_section=skip_section, sep=sep)
+    return s + compare_inp_objects(inp1, inp2, precision=precision, skip_section=skip_section, sep=sep, show_progressbar=show_progressbar)
 
 
-def compare_inp_objects(inp1, inp2, precision=2, skip_section=None, sep='\n' + '#' * 100):
+def compare_inp_objects(inp1, inp2, precision=2, skip_section=None, sep='\n' + '#' * 100, show_progressbar=True):
     s = ''
     sections = set(inp1.keys()) | set(inp2.keys())
 
@@ -155,8 +177,14 @@ def compare_inp_objects(inp1, inp2, precision=2, skip_section=None, sep='\n' + '
         else:
             return len(inp2._original_section_order)
 
-    for section in (progress := tqdm(sorted(sections, key=_sort_by), desc='compare_inp_objects')):
-        progress.set_postfix_str(section)
+    progress = sorted(sections, key=_sort_by)
+
+    if show_progressbar:
+        progress = tqdm(progress, desc='compare_inp_objects')
+
+    for section in progress:
+        if show_progressbar:
+            progress.set_postfix_str(section)
         if skip_section is not None and section in skip_section:
             continue
         if section in [TITLE]:
@@ -185,8 +213,7 @@ def _to_coordinates_list(geo_series):
     return list(zip(geo_series.x, geo_series.y))
 
 
-def _next_nodes():
-    # TODO ...
+def _next_nodes():  # TODO: find nearest node in model
     coords_from = _to_coordinates_list(geometry_from.geometry)
     coords_to = _to_coordinates_list(geometry_to.geometry)
     # distance_matrix = distance.cdist(coords_from, coords_to, 'euclidean')
