@@ -13,7 +13,14 @@ def print_summary(inp):
     if SEC.OPTIONS in inp and "ROUTING_STEP" in inp.OPTIONS:
         print(f'ROUTING_STEP: {inp.OPTIONS["ROUTING_STEP"]}')
     print(f'NODES: {len(nodes_dict(inp)):_d}')
+    print(f'   JUNCTIONS: {len(inp[SEC.JUNCTIONS])}')
+    print(f'   STORAGE: {len(inp[SEC.STORAGE])}')
+    print(f'   OUTFALLS: {len(inp[SEC.OUTFALLS])}')
     print(f'LINKS: {len(links_dict(inp)):_d}')
+    print(f'   CONDUITS: {len(inp[SEC.CONDUITS])}')
+    print(f'   WEIRS: {len(inp[SEC.WEIRS])}')
+    print(f'   OUTLETS: {len(inp[SEC.OUTLETS])}')
+    print(f'   ORIFICES: {len(inp[SEC.ORIFICES])}')
     if SEC.SUBAREAS in inp:
         print(f'SUBCATCHMENTS: {len(inp.SUBAREAS.keys()):_d}')
     if SEC.POLLUTANTS in inp:
