@@ -1,14 +1,22 @@
 import datetime
+import logging
+import warnings
 from typing import Literal
 
-from .._type_converter import infer_type, type2str
+from .._type_converter import infer_type, type2str, get_line_splitter, convert_string
 from ..helpers import InpSectionGeneric, SEP_INP
 from ..section_labels import TITLE, OPTIONS, REPORT, EVAPORATION, TEMPERATURE, MAP, FILES, ADJUSTMENTS, BACKDROP
 
 
+logger = logging.getLogger(__name__)
+
+
 def line_iter(lines):
     if isinstance(lines, str):
+        _split_line = get_line_splitter(lines)
         lines = lines.split('\n')
+    else:
+        _split_line = get_line_splitter('"')
 
     for line in lines:
         line = line.split(';')[0]
@@ -16,7 +24,8 @@ def line_iter(lines):
         if line == '':  # ignore empty and comment lines
             continue
         else:
-            yield line.split()
+
+            yield _split_line(line)
 
 
 class TitleSection(InpSectionGeneric):
@@ -73,6 +82,9 @@ class TitleSection(InpSectionGeneric):
     def __repr__(self):
         return self.txt
 
+    def __str__(self):
+        return self.txt
+
 
 class OptionSection(InpSectionGeneric):
     """
@@ -962,7 +974,7 @@ class ReportSection(InpSectionGeneric):
             if value is None:
                 continue
 
-            if isinstance(value, list) and len(value) > 20:
+            if isinstance(value, (list, tuple, set)) and len(value) > 20:
                 size = len(value)
                 start = 0
                 for end in range(20, size, 20):
@@ -1472,12 +1484,12 @@ class FilesSection(InpSectionGeneric):
             FilesSection: object of the Files-section. Dict-like-object.
         """
         data = cls()
-        for use_save, kind, *fn in line_iter(lines):
+        for use_save, kind, fn in line_iter(lines):
             use_save = use_save.upper()
             kind = kind.upper()
             assert use_save in cls.KEYS._use_or_save
             assert kind in cls.KEYS._possible
-            data[f'{use_save} {kind}'] = ' '.join(fn).strip('"')
+            data[f'{use_save} {kind}'] = convert_string(fn)
         return data
 
     def use(self, kind, filename):
@@ -1573,6 +1585,8 @@ class AdjustmentsSection(InpSectionGeneric):
                 data[key] = [float(i) for i in factors]
             elif key in cls.KEYS._subcatchment:
                 subcatchment, pattern = factors
+                warnings.warn('[ADJUSTMENTS] reading inputfile with subcatchment adjustment is not tested and my fail.')
+                logger.warning('[ADJUSTMENTS] reading inputfile with subcatchment adjustment is not tested and my fail.')
                 data[(key, subcatchment)] = pattern  # todo: might not work, have to test first
                 # if subcatchment not in data:
                 #     data[subcatchment] = {}
