@@ -1,7 +1,9 @@
+from typing import Mapping
+
 from numpy import NaN
 
 from ._identifiers import IDENTIFIERS
-from .._type_converter import to_bool
+from .._type_converter import to_bool, convert_string
 from ..helpers import BaseSectionObject
 from ..section_labels import LID_USAGE, LID_CONTROLS
 
@@ -25,6 +27,7 @@ class LIDControl(BaseSectionObject):
             Name STORAGE Height Vratio Seepage Vclog
             Name DRAIN Coeff Expon Offset Delay
             Name DRAINMAT Thick Vratio Rough
+            Name REMOVALS Pollut PctRemove ...
 
     Examples:
         ::
@@ -54,25 +57,25 @@ class LIDControl(BaseSectionObject):
     Remarks:
         The following table shows which layers are required (x) or are optional (o) for each type of LID process:
 
-        +-----------------------+---------+----------+------+---------+-------+-----------+
-        | LID Type              | Surface | Pavement | Soil | Storage | Drain | Drain Mat |
-        +-----------------------+---------+----------+------+---------+-------+-----------+
-        | Bio-Retention Cell    | x       |          | x    | x       | o     |           |
-        +-----------------------+---------+----------+------+---------+-------+-----------+
-        | Rain Garden           | x       |          | x    |         |       |           |
-        +-----------------------+---------+----------+------+---------+-------+-----------+
-        | Green Roof            | x       |          | x    |         |       | x         |
-        +-----------------------+---------+----------+------+---------+-------+-----------+
-        | Infiltration Trench   | x       |          |      | x       | o     |           |
-        +-----------------------+---------+----------+------+---------+-------+-----------+
-        | Permeable Pavement    | x       | x        | o    | x       | o     |           |
-        +-----------------------+---------+----------+------+---------+-------+-----------+
-        | Rain Barrel           |         |          |      | x       | x     |           |
-        +-----------------------+---------+----------+------+---------+-------+-----------+
-        | Rooftop Disconnection | x       |          |      |         | x     |           |
-        +-----------------------+---------+----------+------+---------+-------+-----------+
-        | Vegetative Swale      | x       |          |      |         |       |           |
-        +-----------------------+---------+----------+------+---------+-------+-----------+
+        +-----------------------+---------+----------+------+---------+-------+-----------+--------------------+
+        | LID Type              | Surface | Pavement | Soil | Storage | Drain | Drain Mat | Pollutant Removals |
+        +-----------------------+---------+----------+------+---------+-------+-----------+--------------------+
+        | Bio-Retention Cell    | x       |          | x    | x       | o     |           | o                  |
+        +-----------------------+---------+----------+------+---------+-------+-----------+--------------------+
+        | Rain Garden           | x       |          | x    |         |       |           |                    |
+        +-----------------------+---------+----------+------+---------+-------+-----------+--------------------+
+        | Green Roof            | x       |          | x    |         |       | x         |                    |
+        +-----------------------+---------+----------+------+---------+-------+-----------+--------------------+
+        | Infiltration Trench   | x       |          |      | x       | o     |           | o                  |
+        +-----------------------+---------+----------+------+---------+-------+-----------+--------------------+
+        | Permeable Pavement    | x       | x        | o    | x       | o     |           | o                  |
+        +-----------------------+---------+----------+------+---------+-------+-----------+--------------------+
+        | Rain Barrel           |         |          |      | x       | x     |           | o                  |
+        +-----------------------+---------+----------+------+---------+-------+-----------+--------------------+
+        | Rooftop Disconnection | x       |          |      |         | x     |           |                    |
+        +-----------------------+---------+----------+------+---------+-------+-----------+--------------------+
+        | Vegetative Swale      | x       |          |      |         |       |           |                    |
+        +-----------------------+---------+----------+------+---------+-------+-----------+--------------------+
 
 
         The equation used to compute flow rate out of the underdrain per unit area of the LID (in in/hr or
@@ -87,33 +90,46 @@ class LIDControl(BaseSectionObject):
         name (str): name assigned to LID process.
         lid_kind (str): One of :attr:`LIDControl.LID_TYPES`.
 
-            - ``BC`` for bio-retention cell
-            - ``RG`` for rain garden; GR for green roof
-            - ``IT`` for infiltration trench
-            - ``PP`` for permeable pavement
-            - ``RB`` for rain barrel
-            - ``RD`` for rooftop disconnection
-            - ``VS`` for vegetative swale.
+            - ``BC`` bio-retention cell
+            - ``RG`` rain garden
+            - ``GR`` green roof
+            - ``IT`` infiltration trench
+            - ``PP`` permeable pavement
+            - ``RB`` rain barrel
+            - ``RD`` rooftop disconnection
+            - ``VS`` vegetative swale
 
-        layer_dict (dict[str, LIDControl.LAYER_TYPES.Surface | LIDControl.LAYER_TYPES.Soil]): Sict of used layers in control.
+        layer_dict (MutableMapping[str, LIDControl.LAYER_TYPES.Surface | LIDControl.LAYER_TYPES.Soil]): Sict of used layers in control.
 
         LID_TYPES: Enum-like for the attribute :attr:`LIDControl.lid_kind` with following members -> {``BC`` | ``RG`` | ``IT`` | ``PP`` | ``RB`` | ``RD`` | ``VS``}
+
+        LAYER_TYPES: Sub-classes for the attribute :attr:`LIDControl.layer_dict` with following members -> {Surface, Soil, Pavement, Storage, Drain, Drainmat, Removals}
     """
-    _identifier = (IDENTIFIERS.name, 'lid_kind')
+    _identifier = IDENTIFIERS.name
     _section_label = LID_CONTROLS
     _table_inp_export = False
 
     class LID_TYPES:
-        BC = 'BC'  # for bio-retention cell
-        RG = 'RG'  # for rain garden
-        GR = 'GR'  # for green roof
-        IT = 'IT'  # for infiltration trench
-        PP = 'PP'  # for permeable pavement
-        RB = 'RB'  # for rain barrel
-        RD = 'RD'  # for rooftop disconnection
-        VS = 'VS'  # for vegetative swale.
+        BC = 'BC'  # bio-retention cell
+        RG = 'RG'  # rain garden
+        GR = 'GR'  # green roof
+        IT = 'IT'  # infiltration trench
+        PP = 'PP'  # permeable pavement
+        RB = 'RB'  # rain barrel
+        RD = 'RD'  # rooftop disconnection
+        VS = 'VS'  # vegetative swale
 
-        _possible = [BC, RG, GR, IT, PP, RB, RD, VS]
+        # Aliases
+        BIO_RETENTION_CELL = BC
+        RAIN_GARDEN = RG
+        GREEN_ROOF = GR
+        INFILTRATION_TRENCH = IT
+        PERMEABLE_PAVEMENT = PP
+        RAIN_BARREL = RB
+        ROOFTOP_DISCONNECTION = RD
+        VEGETATIVE_SWALE = VS
+
+        _possible = {BC, RG, GR, IT, PP, RB, RD, VS}
 
     def __init__(self, name, lid_kind, layer_dict=None):
         """
@@ -136,6 +152,7 @@ class LIDControl(BaseSectionObject):
         self.name = str(name)
         self.lid_kind = lid_kind.upper()  # one of LID_TYPES
         self.layer_dict = {} if layer_dict is None else layer_dict
+        # Using dict instead of list to simplify editing single layer after creation.
 
     @classmethod
     def _convert_lines(cls, multi_line_args):
@@ -148,16 +165,29 @@ class LIDControl(BaseSectionObject):
                     yield last
                 last = cls(name, lid_kind=line[0].upper())
             elif name == last.name:
-                surface_kind = line.pop(0).upper()  # one of SURFACE_TYPES
-                last.layer_dict[surface_kind] = cls.LAYER_TYPES._dict[surface_kind](*line)
+                layer_type = line.pop(0).upper()  # one of LAYER_TYPES
+                layer = cls.LAYER_TYPES._dict[layer_type](*line)
+                last.add_layer(layer)
+                # last.layer_dict[layer_type] = layer
         yield last
 
     class LAYER_TYPES:
+        """Layer types to add to LID controls."""
         class Surface(BaseSectionObject):
             _LABEL = 'SURFACE'
 
-            def __init__(self, StorHt, VegFrac, Rough, Slope, Xslope):
+            def __init__(self, StorHt, VegFrac=0, Rough=0, Slope=0, Xslope=0):
                 """
+
+                Used:
+                    bio-retention cell
+                    rain garden
+                    green roof
+                    infiltration trench
+                    permeable pavement
+                    rooftop disconnection
+                    vegetative swale
+
                 Args:
                     StorHt (float): when confining walls or berms are present this is the maximum depth to which water can
                         pond above the surface of the unit before overflow occurs (in inches or mm). For LIDs that
@@ -187,6 +217,12 @@ class LIDControl(BaseSectionObject):
 
             def __init__(self, Thick, Por, FC, WP, Ksat, Kcoeff, Suct):
                 """
+                Used:
+                    bio-retention cell
+                    rain garden
+                    green roof
+                    permeable pavement (only optional)
+
                 Args:
                     Thick (float): thickness of the soil layer (inches or mm).
                     Por (float): soil porosity (volume of pore space relative to total volume).
@@ -212,6 +248,9 @@ class LIDControl(BaseSectionObject):
 
             def __init__(self, Thick, Vratio, FracImp, Perm, Vclog, regeneration_interval=NaN, regeneration_fraction=NaN):
                 """
+                Used:
+                    permeable pavement
+
                 Args:
                     Thick (float): thickness of the pavement layer (inches or mm).
                     Vratio (float): void ratio (volume of void space relative to the volume of solids in the
@@ -223,8 +262,7 @@ class LIDControl(BaseSectionObject):
                     Perm (float): permeability of the concrete or asphalt used in continuous systems or
                         hydraulic conductivity of the fill material (gravel or sand) used in modular
                         systems (in/hr or mm/hr).
-                    Vclog (float): number of pavement layer void volumes of runoff treated it takes to
-                        completely clog the pavement. Use a value of 0 to ignore clogging.
+                    Vclog (float): number of pavement layer void volumes of runoff treated it takes to completely clog the pavement. Use a value of 0 to ignore clogging.
                 """
                 self.Thick = float(Thick)
                 self.Vratio = float(Vratio)
@@ -240,6 +278,12 @@ class LIDControl(BaseSectionObject):
 
             def __init__(self, Height, Vratio, Seepage, Vclog, Covrd=True):
                 """
+                Used:
+                    bio-retention cell
+                    infiltration trench
+                    permeable pavement
+                    rain barrel
+
                 Args:
                     Height (float): thickness of the storage layer or height of a rain barrel (inches or mm).
                     Vratio (float): void ratio (volume of void space relative to the volume of solids in the
@@ -265,6 +309,14 @@ class LIDControl(BaseSectionObject):
 
             def __init__(self, Coeff, Expon, Offset, Delay, open_level=NaN, close_level=NaN, Qcurve=NaN):
                 """
+
+                Used:
+                    bio-retention cell (only optional)
+                    infiltration trench (only optional)
+                    permeable pavement (only optional)
+                    rain barrel
+                    rooftop disconnection
+
                 Args:
                     Coeff (float): coefficient C that determines the rate of flow through the drain as a
                         function of height of stored water above the drain bottom. For Rooftop
@@ -299,6 +351,9 @@ class LIDControl(BaseSectionObject):
 
             def __init__(self, Thick, Vratio, Rough):
                 """
+                Used:
+                    GreenRoofs
+
                 Args:
                     Thick (float): thickness of the drainage mat (inches or mm).
                     Vratio (float): ratio of void volume to total volume in the mat.
@@ -313,9 +368,19 @@ class LIDControl(BaseSectionObject):
 
             def __init__(self, *pollutant_removal_rate):
                 """
+                Pollutant removals.
+
+                Only when pollutants are defined.
+
+                Used:
+                    bio-retention cell
+                    infiltration trench
+                    permeable pavement
+                    rain barrel
+
                 Args:
-                    Pollut (): name of a pollutant
-                    Rmvl (): the percent removal the LID achieves for the pollutant (several pollutant
+                    Pollut (str): name of a pollutant
+                    Rmvl (flaat): the percent removal the LID achieves for the pollutant (several pollutant
                         removals can be placed on the same line or specified in separate REMOVALS
                         lines).
                 """
@@ -329,10 +394,29 @@ class LIDControl(BaseSectionObject):
         DRAINMAT = Drainmat._LABEL
         REMOVALS = Removals._LABEL
 
-        _possible = [SURFACE, SOIL, PAVEMENT, STORAGE, DRAIN, DRAINMAT, REMOVALS]
+        _possible = {SURFACE, SOIL, PAVEMENT, STORAGE, DRAIN, DRAINMAT, REMOVALS}
 
         _dict = {x._LABEL: x for x in (Surface, Soil, Pavement, Storage, Drain, Drainmat, Removals)}
 
+    def add_layer(self, layer):
+        """
+        Add a new layer to the LID control. Or overwrite the existing one.
+
+        Args:
+            layer (LIDControl.LAYER_TYPES.Surface | LIDControl.LAYER_TYPES.Soil | LIDControl.LAYER_TYPES.Pavement | LIDControl.LAYER_TYPES.Storage | LIDControl.LAYER_TYPES.Drain | LIDControl.LAYER_TYPES.Drainmat | LIDControl.LAYER_TYPES.Removals): layer object.
+        """
+        self.layer_dict[layer._LABEL] = layer
+
+    def add_layer_as_dict(self, layer_type, kwargs):
+        """
+        Add a new layer to the LID control. Or overwrite the existing one.
+
+        Args:
+            layer_type (str): label of the layer. One of :attr:`LIDControl.LAYER_TYPES._possible` -> {``SURFACE``| ``SOIL``| ``PAVEMENT``| ``STORAGE``| ``DRAIN``| ``DRAINMAT``| ``REMOVALS``}
+            kwargs (dict): keyword arguments of the new layer.
+        """
+        self.add_layer(self.LAYER_TYPES._dict[layer_type](**kwargs))
+
     def to_inp_line(self):
         s = f'{self.name} {self.lid_kind}\n'
         for layer, l in self.layer_dict.items():
@@ -462,6 +546,6 @@ class LIDUsage(BaseSectionObject):
         self.saturation_init = float(saturation_init)
         self.impervious_portion = float(impervious_portion)
         self.route_to_pervious = int(route_to_pervious)
-        self.fn_lid_report = fn_lid_report
+        self.fn_lid_report = convert_string(fn_lid_report)
         self.drain_to = drain_to
         self.from_pervious = from_pervious
