@@ -249,7 +249,8 @@ class Outlet(_Link):
 
     @property
     def curve_name(self):
-        return self.curve_description is isinstance(self.curve_description, str)
+        if isinstance(self.curve_description, str):
+            return self.curve_description
 
     def _functional_init(self, coefficient, exponent, has_flap_gate=False):
         """
@@ -417,4 +418,4 @@ class Weir(_Link):
         self.discharge_coefficient_end = float(discharge_coefficient_end)
         self.can_surcharge = to_bool(can_surcharge)
         self.road_width = float(road_width)
-        self.road_surface = str(road_surface)
+        self.road_surface = road_surface if isinstance(road_surface, str) and (road_surface.lower() != 'nan') else road_surface
