@@ -1,3 +1,5 @@
 from .out import read_out_file, SwmmOutput, out2frame
 from .definitions import VARIABLES, OBJECTS
 from . import definitions as OUT
+from . import parquet_helpers
+parq = parquet_helpers
\ No newline at end of file
