@@ -37,9 +37,12 @@ _NODES_TYPES = ['JUNCTION', 'OUTFALL', 'STORAGE', 'DIVIDER']
 _LINK_TYPES = ['CONDUIT', 'PUMP', 'ORIFICE', 'WEIR', 'OUTLET']
 
 
+_ERROR_MESSAGE_PREFIX = '*   '
+
+
 class SwmmExtractValueError(Exception):
     def __init__(self, message):
-        super().__init__(f'\n*\n*   {message}\n*\n')
+        super().__init__(f'\n*\n{_ERROR_MESSAGE_PREFIX}{message}\n*\n')
 
 
 class SwmmOutExtractWarning(UserWarning):
@@ -96,7 +99,7 @@ class SwmmOutExtract(BinaryReader):
             _pos_start_labels,  # starting file position of ID names
             _pos_start_input,  # starting file position of input data
             _pos_start_output,  # starting file position of output data
-            _n_periods,  # Number of reporting periods
+            self.n_periods,  # Number of reporting periods
             error_code,
             magic_num_end,
         ) = self._next(6)
@@ -109,12 +112,13 @@ class SwmmOutExtract(BinaryReader):
         # ____
         # check errors
         if magic_num_start != _MAGIC_NUMBER:
-            raise SwmmExtractValueError('Beginning magic number incorrect.')
+            raise SwmmExtractValueError(f'Beginning magic number incorrect.\n'
+                                        f'{_ERROR_MESSAGE_PREFIX}Potential causes for this issue include either the file ({self.filename}) not being a binary SWMM output file\n'
+                                        f'{_ERROR_MESSAGE_PREFIX}or the output file being corrupted.')
 
         if magic_num_end != _MAGIC_NUMBER:
             warn('Ending magic number incorrect.', SwmmOutExtractWarning)
             # raise SwmmExtractValueError('Ending magic number incorrect.')
-            _n_periods = 0
             self.run_failed = True
         elif error_code != 0:
             warn(f'Error code "{error_code}" in output file indicates a problem with the run.', SwmmOutExtractWarning)
@@ -216,7 +220,7 @@ class SwmmOutExtract(BinaryReader):
             return ErrorCode;
         }
         """
-        _base_date = datetime.datetime(1899, 12, 30)
+        self._base_date = datetime.datetime(1899, 12, 30)
         _offset_start_td = datetime.timedelta(days=self._next(dtype='d'))
         # self.start_date_ = _base_date + _offset_start_td
         self.report_interval = datetime.timedelta(seconds=self._next())
@@ -239,11 +243,10 @@ class SwmmOutExtract(BinaryReader):
         _factor = (_offset_first_index_td - _offset_start_td) / self.report_interval
 
         # get real start date of the timeseries
-        self.start_date = _base_date + _offset_start_td + self.report_interval * int(_factor)
+        self.start_date = self._base_date + _offset_start_td + self.report_interval * int(_factor)
 
         # ____
-        self.n_periods = _n_periods
-        if _n_periods == 0:
+        if magic_num_end != _MAGIC_NUMBER:
             self._infer_n_periods()
             warn('Infer time periods of the output file due to a corrupt SWMM .out-file.', SwmmOutExtractWarning)
 
@@ -251,9 +254,6 @@ class SwmmOutExtract(BinaryReader):
             warn('There are zero time periods in the output file.', SwmmOutExtractWarning)
             # raise SwmmExtractValueError('There are zero time periods in the output file.')
 
-    def __repr__(self):
-        return f'SwmmOutExtract(file="{self.filename}")'
-
     def _infer_bytes_per_period(self):
         """
         Calculate the bytes for each time period when reading the computed results
@@ -268,21 +268,7 @@ class SwmmOutExtract(BinaryReader):
         _bytes_per_period *= _RECORDSIZE
         return _bytes_per_period
 
-    def _get_selective_results(self, columns, processes=1, show_progress=True):
-        """
-        get results of selective columns in .out-file
-
-        this function is due to its iterative reading slow,
-        but has it advantages with out-files with many columns (>1000) and fewer time-steps
-
-        Args:
-            columns (list[tuple]): list of column identifier tuple with [(kind, label, variable), ...]
-            processes (int): number of parallel processes.
-            show_progress (bool): show a progress bar.
-
-        Returns:
-            dict[str, list]: dictionary where keys are the column names ('/' as separator) and values are the list of result values
-        """
+    def _get_labels_and_offsets(self, columns):
         n_vars_subcatch = len(self.variables[OBJECTS.SUBCATCHMENT])
         n_vars_node = len(self.variables[OBJECTS.NODE])
         n_vars_link = len(self.variables[OBJECTS.LINK])
@@ -292,10 +278,10 @@ class SwmmOutExtract(BinaryReader):
         n_links = len(self.labels[OBJECTS.LINK])
 
         offset_list = []
-        values = {}
+        label_list = []
 
         for kind, label, variable in columns:
-            values['/'.join([kind, label, variable])] = []
+            label_list.append('/'.join([kind, label, variable]))
 
             index_kind = OBJECTS.LIST_.index(kind)
             index_variable = self.variables[kind].index(variable)
@@ -310,71 +296,135 @@ class SwmmOutExtract(BinaryReader):
                 4: (n_subcatch * n_vars_subcatch +
                     n_nodes * n_vars_node +
                     n_links * n_vars_link)
-            }[index_kind])*_RECORDSIZE)
+            }[index_kind]) * _RECORDSIZE)
 
-        # offset_list = [o*_RECORDSIZE for o in offset_list]
-        # cols = list(values.keys())
-        # cols_sorted = sorted(cols, key=lambda e: offset_list[cols.index(e)])
-        # offset_sorted = sorted(offset_list)
-        # iter_label_offset = tuple(zip(cols_sorted, offset_sorted))
-        iter_label_offset = tuple(zip(values.keys(), offset_list))
+        return label_list, offset_list
 
-        variable = range(self._pos_start_output,  # start
-                         self._pos_start_output + self.n_periods * self._bytes_per_period,  # stop
-                         self._bytes_per_period)
+    def _get_start_position_and_periods(self, start=None, end=None):
+        if all([i is None for i in [start, end]]):
+            return self._pos_start_output, self.n_periods
 
-        def func(out, period_offset):
-            for label, offset in iter_label_offset:
-                out._set_position(offset + period_offset)
-                values[label].append(out._next_float())
-
-        # for period_offset in tqdm(range(self._pos_start_output,  # start
-        #                                 self._pos_start_output + self.n_periods * self._bytes_per_period,  # stop
-        #                                 self._bytes_per_period),
-        #                           desc=f'{repr(self)}.get_selective_results(n_cols={len(columns)})'):  # step
-        #     # period_offset = self.pos_start_output + period * self.bytes_per_period
-        #     for label, offset in iter_label_offset:
-        #         self._set_position(offset + period_offset)
-        #         values[label].append(self._next_float())
-
-        # -------------
-        from functools import partial
-        from itertools import cycle
-
-        if show_progress:
-            from tqdm.auto import tqdm
+        if start is None:
+            i_period_start = 0
         else:
-            tqdm = lambda _, desc=None: _
-
-        if processes == 1:
-            for period_offset in tqdm(variable, desc=f'{repr(self)}.get_selective_results(n_cols={len(columns)})'):
-                func(self, period_offset)
+            i_period_start = (start - self.start_date) / self.report_interval
 
+        if end is None:
+            i_period_end = self.n_periods - i_period_start
         else:
-            from multiprocessing.dummy import Pool
+            i_period_end = (end - self.start_date) / self.report_interval
 
-            files = [self.copy() for _ in range(processes)]
-            pool = Pool(processes)
-            for _ in tqdm(pool.starmap(partial(func), zip(cycle(files), variable)), total=len(variable), desc=f'{repr(self)}.get_selective_results(n_cols={len(columns)}, processes={processes})'):
-                pass
-        # -------------
+        n_periods = i_period_end - i_period_start + 1
+        pos_start_output = self._pos_start_output + i_period_start * self._bytes_per_period
+        return int(pos_start_output), int(n_periods)
 
-        return values
+    def _get_selective_results(self, columns, show_progress=True, start=None, end=None):
+        """
+        Get results of selective columns in .out-file.
+
+        This function is due to its iterative reading slow,
+        but has its advantages with out-files with many columns (>1000) and fewer time-steps
+
+        Args:
+            columns (list[tuple]): list of column identifier tuple with [(kind, label, variable), ...]
+            show_progress (bool): show a progress bar.
+            start (datetime): start datetime for the period to read.
+            end (datetime): end datetime for the period to read.
+
+        Returns:
+            dict[str, list]: dictionary where keys are the column names ('/' as separator) and values are the list of result values
+        """
+        progress_desc = f'{repr(self)}.get_selective_results(n_cols={len(columns)})'
+
+        label_list, offset_list = self._get_labels_and_offsets(columns)
+        pos_start_output, n_periods = self._get_start_position_and_periods(start, end)
+
+        # ---
+        # from ._basic_selective_results import _get_selective_results  # cpython
+        from ._basic_selective_results_python import _get_selective_results
+        v = _get_selective_results(self.fp, label_list, offset_list, pos_start_output,
+                                   n_periods, self._bytes_per_period, progress_desc, show_progress)
+        return v
 
     def _infer_n_periods(self):
+        """
+        Internal function to infer the number of periods written in the out-file.
+
+        This is needed when the outfile was not generated properly.
+
+        Sets the attribute n_periods.
+        """
+        # print('start _infer_n_periods')
         not_done = True
+        # print(f'{self.filename.stat().st_size:_d}')
+        # 192_159_825_920 filesize
+        # 179_517_985_920
+        #       2_409_120 approx. index size
+        #          18_627 column size
+        #          91_772 already read bytes
+        #          74_516 bytes per period
+        # print(self.number_columns)
+        # print(self._bytes_per_period)
+
+        last_offset = self.fp.seek(-6*_RECORDSIZE, SEEK_END)
+        est_n_periods = int((last_offset - self._pos_start_output) / self._bytes_per_period)-1
+        # self.n_periods
+
+        step = int(est_n_periods / 4)
         period = 0
+
+        # 1289386
+        # 2311667
+        # print(f'{est_n_periods=} | {step=}')
+
+        # self.start_date
+
+        # start_date = self._base_date + _offset_start_td + self.report_interval * int(_factor)
+
+        # self.start_date + est_n_periods * self.report_interval
+
+        # self.fp.tell()
+        import math
+
+        # period = est_n_periods-1
+
+        first_failed = False
+        i = 1
         while not_done:
+            dt_theory = self.start_date + period * self.report_interval
+            # print(f'___\n{i}: {period=}/{est_n_periods} | {str(dt_theory)=} | {step=}')
+            # print(f'pos={self._pos_start_output + period * self._bytes_per_period}')
             self.fp.seek(self._pos_start_output + period * self._bytes_per_period, SEEK_SET)
             try:
-                dt = self._next(dtype='d')
-                # print(dt)
-                # print(datetime.datetime(1899, 12, 30) + datetime.timedelta(days=dt))
-                period += 1
-            except:
-                not_done = False
+                dt = self._next(dtype='d')  # unpack requires a buffer of 8 bytes
 
-        self.n_periods = period - 1
+                # print(f'{dt=}')
+                # if dt is very large, python will raise an error
+                # dt can also be near zero, what is also an error, but will not raise
+
+                # print(self._base_date + datetime.timedelta(days=dt))
+                # print(f'tell={self.fp.tell()}')
+                if dt < 0.00000001:
+                    raise EOFError('dt < 0.00000001')
+
+                if first_failed:
+                    step = math.ceil(step/2)
+                period += step
+
+            except Exception as e:
+                if not first_failed:
+                    first_failed = True
+
+                # print('###', e, f'tell={self.fp.tell()}')
+
+                if step <= 1:
+                    break
+
+                step = math.ceil(step/2)
+                period -= step
+            i += 1
+
+        self.n_periods = period
 
     def copy(self):
         new = type(self)(self.filename, skip_init=True)
