@@ -8,14 +8,18 @@ __license__ = "MIT"
 import datetime
 import warnings
 from itertools import product
-from numpy import dtype, fromfile, frombuffer
-from pandas import date_range, DataFrame, MultiIndex, Index
+from pathlib import Path
+
+import numpy as np
+import pandas as pd
+
 from pandas._libs import OutOfBoundsDatetime
 
 from .extract import SwmmOutExtract
 from .definitions import OBJECTS, VARIABLES
 
 from . import parquet_helpers as parquet
+from .helpers import drop_useless_column_levels
 
 
 class SwmmOutputWarning(UserWarning):
@@ -53,7 +57,8 @@ class SwmmOutput(SwmmOutExtract):
         variables (dict[str, list]): variables per object-type inclusive the pollutants.
         fp (file-like): Stream of the open file.
     """
-    filename: str = ...
+    filename: Path = ...
+
     def __init__(self, filename, skip_init=False, encoding=''):
         """
         Read a SWMM-output-file (___.out).
@@ -71,13 +76,10 @@ class SwmmOutput(SwmmOutExtract):
             return
 
         # the main datetime index for the results
-        try:
-            self.index = date_range(self.start_date, periods=self.n_periods, freq=self.report_interval)
-        except OutOfBoundsDatetime:
-            self.index = Index([self.start_date + self.report_interval * i for i in range(self.n_periods)])
+        self.index = self._get_index()
 
-    def __repr__(self):
-        return f'SwmmOutput(file="{self.filename}")'
+    def __enter__(self):
+        return self
 
     def _get_dtypes(self):
         """
@@ -114,24 +116,78 @@ class SwmmOutput(SwmmOutExtract):
             columns += list(product([kind], self.labels[kind], self.variables[kind]))
         return columns
 
-    def to_numpy(self):
+    def _get_index(self, start=None, end=None):
+        if start is None:
+            start = self.start_date
+
+        # n_periods = number of time-steps (rows) to read (including start time-step (row))
+        if end is None:
+            n_periods = int(self.n_periods - (start - self.start_date) / self.report_interval)
+        else:
+            n_periods = int((end - start) / self.report_interval) + 1
+
+        try:
+            return pd.date_range(start, periods=n_periods, freq=self.report_interval)
+        except OutOfBoundsDatetime:
+            warnings.warn('Can not create a pandas.DatetimeIndex in given date-range. Default to pandas.Index.')
+            return pd.Index([start + self.report_interval * i for i in range(n_periods)])
+
+    def to_numpy(self, ignore_filesize=False, start=None, end=None):
         """
         Convert all data to a numpy-array.
 
+        Args:
+            ignore_filesize (bool): hide a warning when reading a huge output-file.
+            start (datetime): start datetime for the period to read.
+            end (datetime): end datetime for the period to read.
+
         Returns:
             numpy.ndarray: all data
         """
         if self._data is None:
-            types = [('datetime', 'f8')]
-            types += list(map(lambda i: ('/'.join(i), 'f4'), self._columns_raw))
-            self.fp.seek(self._pos_start_output, 0)
-            try:
-                self._data = fromfile(self.fp, dtype=dtype(types))
-            except:
-                self._data = frombuffer(self.fp.read1(), dtype=dtype(types), count=self.n_periods)
+
+            if not ignore_filesize and isinstance(self.filename, Path):
+                stats_file = self.filename.lstat()
+                filesize = stats_file.st_size  # in bytes
+                filesize_gb = filesize*1e-9
+                if filesize_gb > 6:
+                    warnings.warn(f"You are attempting to read a large output file ({filesize_gb:0.3f} GB), "
+                                  f"which could potentially lead to memory overflow. "
+                                  f"If you are only interested in a specific portion of the file, "
+                                  f"please utilize the .get_part() function with the slim=True parameter. "
+                                  f"This way, only the necessary data is read from the file. "
+                                  f"To disregard and suppress this message, use the ignore_filesize=True parameter.", SwmmOutputWarning)
+
+            types = [('datetime', 'f8')] + [('/'.join(i), 'f4') for i in self._columns_raw]
+
+            pos_start_output, n_periods = self._get_start_position_and_periods(start, end)
+
+            self.fp.seek(pos_start_output, 0)
+
+            if isinstance(self.filename, str) and (self.filename == '<stream>'):
+                array = np.frombuffer(self.fp.read1(), dtype=np.dtype(types), count=n_periods)
+            else:
+                try:
+                    array = np.fromfile(self.fp, dtype=np.dtype(types), count=n_periods)
+                except:
+                    array = np.frombuffer(self.fp.read1(), dtype=np.dtype(types), count=n_periods)
+
+            if all([i is None for i in [start, end]]):
+                # if the data is not sliced, save it as an attribute
+                self._data = array
+
+            return array
+
+        # if the attribute self._data is already set
+        if not all([i is None for i in [start, end]]):
+            # slice data
+            i_start = (self.index < start).sum()
+            i_end = (self.index > end).sum()
+            return self._data[i_start:-i_end]
+
         return self._data
 
-    def to_frame(self):
+    def to_frame(self, ignore_filesize=False):
         """
         Convert all the data to a pandas-DataFrame.
 
@@ -139,15 +195,18 @@ class SwmmOutput(SwmmOutExtract):
             This function may take a long time if the out-file has with many objects (=columns).
             If you just want the data of a few columns use :meth:`SwmmOutput.get_part` instead.
 
+        Args:
+            ignore_filesize (bool): hide a warning when reading a huge output-file.
+
         Returns:
             pandas.DataFrame: data
         """
         if self._frame is None:
-            self._frame = self._to_pandas(self.to_numpy())
+            self._frame = self._to_pandas(self.to_numpy(ignore_filesize=ignore_filesize))
             del self._frame['datetime']
         return self._frame
 
-    def get_part(self, kind=None, label=None, variable=None, slim=False, processes=1, show_progress=True):
+    def get_part(self, kind=None, label=None, variable=None, slim=False, show_progress=True, ignore_filesize=False, start=None, end=None):
         """
         Get specific columns of the data.
 
@@ -202,18 +261,24 @@ class SwmmOutput(SwmmOutExtract):
             slim (bool): set to ``True`` to speedup the code if there are a lot of objects and just few time-steps in the out-file.
             processes (int): number of parallel processes for the slim-reading.
             show_progress (bool): show a progress bar for the slim-reading.
+            ignore_filesize (bool): hide a warning when reading a huge output-file.
+            start (datetime): start datetime for the period to read.
+            end (datetime): end datetime for the period to read.
 
         Returns:
             pandas.DataFrame | pandas.Series: Filtered data.
                 (return Series if only one column is selected otherwise return a DataFrame)
         """
         columns = self._filter_part_columns(kind, label, variable)
-        if slim:
-            values = self._get_selective_results(columns, processes=processes, show_progress=show_progress)
-        else:
-            values = self.to_numpy()[list(map('/'.join, columns))]
 
-        return self._to_pandas(values, drop_useless=True)
+        if slim:
+            values = self._get_selective_results(columns, show_progress=show_progress, start=start, end=end)
+        else:
+            values = self.to_numpy(ignore_filesize=ignore_filesize, start=start, end=end)[list(map('/'.join, columns))]
+
+        index = self._get_index(start, end)
+
+        return self._to_pandas(values, index, drop_useless=True)
 
     def _filter_part_columns(self, kind=None, label=None, variable=None):
         """
@@ -252,9 +317,9 @@ class SwmmOutput(SwmmOutExtract):
             columns += list(product([k], _filter(label, self.labels[k], f'{k} label'), _filter(variable, self.variables[k], f'{k} variable')))
         return columns
 
-    def _to_pandas(self, data, drop_useless=False):
+    def _to_pandas(self, data, index: pd.DatetimeIndex = None, drop_useless=False):
         """
-        convert interim results to pandas DataFrame or Series
+        Convert interim results to pandas DataFrame or Series.
 
         Args:
             data (dict, numpy.ndarray): timeseries data of swmm out file
@@ -263,26 +328,37 @@ class SwmmOutput(SwmmOutExtract):
         Returns:
             (pandas.DataFrame | pandas.Series): pandas Timerseries of data
         """
-        if isinstance(data, dict):
-            if not bool(data):
-                return DataFrame()
-            df = DataFrame.from_dict(data).set_index(self.index)
-        else:
-            if data.size == 0:
-                return DataFrame()
+        if index is None:
+            index = self.index
 
-            if data.shape[0] != self.index.size:
-                data = data[:self.index.size]
+        if (isinstance(data, np.ndarray) and (data.size == 0)) or (isinstance(data, dict) and not bool(data)):  # successful for dict and np.array
+            return pd.DataFrame()
 
-            df = DataFrame(data, index=self.index, dtype=float)
+        df = pd.DataFrame(data, index=index, dtype=float)
+
+        # if isinstance(data, dict):  # slim=True
+        #     df = pd.DataFrame.from_dict(data).set_index(index)
+        #
+        # else:  # slim=False - default - numpy.array
+        #     data: np.ndarray
+        #
+        #     # if data.shape[0] != index.size:
+        #     #     warnings.warn('index length does not match data length -> slice data')
+        #     #     data = data[:index.size]
+        #
+        #     pd.DataFrame.from_dict(data).set_index(index)
+        #
+        #     df = pd.DataFrame(data, index=index, dtype=float)
 
         # -----------
         if df.columns.size == 1:
             return df.iloc[:, 0]
         # -----------
-        df.columns = MultiIndex.from_tuples([col.split('/') for col in df.columns])
+        # create multi-index-columns
+        df.columns = pd.MultiIndex.from_tuples([col.split('/') for col in df.columns])
+
         if drop_useless:
-            df.columns = df.columns.droplevel([i for i, l in enumerate(df.columns.levshape) if l == 1])
+            drop_useless_column_levels(df)
 
         return df
 
@@ -296,7 +372,71 @@ class SwmmOutput(SwmmOutExtract):
 
         Read parquet files with :func:`swmm_api.output_file.parquet.read` to get the original column-name-structure.
         """
-        parquet.write(self.to_frame(), self.filename.replace('.out', '.parquet'))
+        # flow_unit, model_properties, run_failed, swmm_version as .attr
+        frame = self.to_frame()
+        frame.attrs = {
+            'flow_unit': self.flow_unit,
+            'model_properties': self.model_properties,
+            'run_failed': self.run_failed,
+            'swmm_version': self.swmm_version,
+        }
+        parquet.write(frame, self.filename.with_suffix('.parquet'))
+
+    def to_parquet_chunks(self, fn, rows_at_a_time=1000, show_progress=True, kind=None, label=None, variable=None, slim=False):
+        import pyarrow as pa
+        import pyarrow.parquet as pq
+
+        types = np.dtype([('datetime', 'f8')] + [('/'.join(i), 'f4') for i in self._columns_raw])
+        self.fp.seek(self._pos_start_output, 0)
+
+        parq_writer = None
+        columns = self._filter_part_columns(kind, label, variable)
+        use_columns = ['datetime'] + ['/'.join(c) for c in columns]
+
+        # ---
+        if show_progress:
+            import tqdm
+            iterator = tqdm.trange(0, self.n_periods, rows_at_a_time)
+        else:
+            iterator = range(0, self.n_periods, rows_at_a_time)
+
+        # ---
+        for i in iterator:
+            # print(self.fp.tell())
+            if slim:
+                progress_desc = f'{repr(self)}.get_selective_results(n_cols={len(columns)})'
+
+                label_list, offset_list = self._get_labels_and_offsets(columns)
+
+                from ._basic_selective_results import _get_selective_results
+
+                _pos_start_output = self._pos_start_output + i * self._bytes_per_period
+
+                data = _get_selective_results(self.fp, label_list, offset_list, _pos_start_output,
+                                              rows_at_a_time, self._bytes_per_period, progress_desc)
+
+                df = pd.DataFrame(data)
+
+                df.index = pd.date_range(self.start_date + i*self.report_interval, periods=rows_at_a_time, freq=self.report_interval)
+
+            else:
+                data = np.fromfile(self.fp, dtype=types, count=rows_at_a_time)[use_columns]
+
+                df = pd.DataFrame(data)
+
+                df.index = (pd.Timedelta(days=1) * df.pop('datetime') + self._base_date).dt.round('s')
+
+            table = pa.Table.from_pandas(df)
+
+            # for the first chunk of records
+            if parq_writer is None:
+                # create a parquet write object giving it an output file
+                parq_writer = pq.ParquetWriter(fn, table.schema, compression='brotli')
+            parq_writer.write_table(table)
+
+        # close the parquet writer
+        if parq_writer:
+            parq_writer.close()
 
     # def to_netcdf(self, fn_nc):
     #     import xarray as xr
