@@ -7,11 +7,14 @@ __license__ = "MIT"
 
 import datetime
 import os.path
+import warnings
 
 import pandas as pd
 
-from .helpers import (_get_title_of_part, _remove_lines, _part_to_frame, _continuity_part_to_dict, ReportUnitConversion,
-                      _routing_part_to_dict, _quality_continuity_part_to_dict, )
+from .helpers import (_get_title_of_part, _remove_lines, _part_to_frame,
+                      _continuity_part_to_dict, ReportUnitConversion,
+                      _routing_part_to_dict, _quality_continuity_part_to_dict,
+                      _transect_street_shape_converter, _options_part_to_dict, )
 from .._io_helpers._encoding import get_default_encoding
 from .._io_helpers._read_txt import read_txt_file
 from ..input_file.helpers import natural_keys
@@ -63,12 +66,15 @@ class SwmmReport:
         self._link_summary = None
         self._cross_section_summary = None
         self._transect_summary = None
+        self._street_summary = None
+        self._shape_summary = None
 
         self._runoff_quantity_continuity = None
         self._flow_routing_continuity = None
         self._groundwater_continuity = None
         self._quality_routing_continuity = None
         self._runoff_quality_continuity = None
+        self._rainfall_dependent_ii = None
 
         self._highest_continuity_errors = None
         self._time_step_critical_elements = None
@@ -98,6 +104,8 @@ class SwmmReport:
 
         self._pumping_summary = None
 
+        self._street_flow_summary = None
+
         self._lid_control_summary = None
         self._lid_performance_summary = None
 
@@ -136,7 +144,8 @@ class SwmmReport:
         txt = read_txt_file(self._filename, encoding=self._encoding)
         lines = txt.split('\n')
 
-        if not lines:
+        if not txt:
+            warnings.warn(f'SWMM-report-file ({self._filename}) is empty.')
             return
 
         self._raw_parts['Simulation Infos'] = '\n'.join(lines[-3:])
@@ -165,7 +174,7 @@ class SwmmReport:
     def _get_converted_part(self, key):
         if key not in self._converted_parts:
             if key not in self._raw_parts:
-                return ''
+                return
             self._converted_parts[key] = _remove_lines(self._raw_parts[key], title=True, empty=False)
 
         return self._converted_parts[key]
@@ -190,33 +199,7 @@ class SwmmReport:
         """
         if self._analysis_options is None:
             p = self._get_converted_part('Analysis Options')
-
-            res = {}
-            last_key = None
-            last_initial_spaces = 0
-
-            for line in p.split('\n'):
-                initial_spaces = len(line) - len(line.lstrip())
-
-                if '..' in line:
-                    key = line[:line.find('..')].strip()
-                    value = line[line.rfind('..') + 2:].strip()
-
-                    if last_initial_spaces > initial_spaces:
-                        last_key = None
-
-                    if last_key:
-                        res[last_key].update({key: value})
-                    else:
-                        res[key] = value
-
-                    last_initial_spaces = initial_spaces
-
-                elif line.strip():
-                    last_key = line.replace(':', '').strip()
-                    res[last_key] = {}
-
-            self._analysis_options = res
+            self._analysis_options = _options_part_to_dict(p)
         return self._analysis_options
 
     @property
@@ -227,6 +210,8 @@ class SwmmReport:
         Returns:
             str: flow unit
         """
+        if self.analysis_options is None:
+            return
         if 'Flow Units' in self.analysis_options:
             return self.analysis_options['Flow Units']
 
@@ -253,33 +238,7 @@ class SwmmReport:
         """
         if self._element_count is None:
             p = self._get_converted_part('Element Count')
-
-            res = {}
-            last_key = None
-            last_initial_spaces = 0
-
-            for line in p.split('\n'):
-                initial_spaces = len(line) - len(line.lstrip())
-
-                if '..' in line:
-                    key = line[:line.find('..')].strip()
-                    value = line[line.rfind('..') + 2:].strip()
-
-                    if last_initial_spaces > initial_spaces:
-                        last_key = None
-
-                    if last_key:
-                        res[last_key].update({key: value})
-                    else:
-                        res[key] = value
-
-                    last_initial_spaces = initial_spaces
-
-                elif line.strip():
-                    last_key = line.replace(':', '').strip()
-                    res[last_key] = {}
-
-            self._element_count = res
+            self._element_count = _options_part_to_dict(p)
         return self._element_count
 
     @property
@@ -321,6 +280,19 @@ class SwmmReport:
             self._groundwater_continuity = _continuity_part_to_dict(p)
         return self._groundwater_continuity
 
+    @property
+    def rainfall_dependent_ii(self):
+        """
+        Rainfall Dependent I/I
+
+        Returns:
+            dict[str, dict[str, float]]: Rainfall Dependent I/I
+        """
+        if self._rainfall_dependent_ii is None:
+            p = self._raw_parts.get('Rainfall Dependent I/I', None)
+            self._rainfall_dependent_ii = _continuity_part_to_dict(p)
+        return self._rainfall_dependent_ii
+
     @property
     def quality_routing_continuity(self):
         """
@@ -396,7 +368,6 @@ class SwmmReport:
         """
         if self._node_summary is None:
             p = self._get_converted_part('Node Summary')
-            # p = '-'*10 + '\n' + p
             self._node_summary = _part_to_frame(p)
         return self._node_summary
 
@@ -410,10 +381,9 @@ class SwmmReport:
         """
         if self._link_summary is None:
             p = self._get_converted_part('Link Summary')
-            # p = '-'*10 + '\n' + p
-            p = p.replace('From Node', 'FromNode')
-            p = p.replace('To Node', 'ToNode')
-            self._link_summary = _part_to_frame(p)
+            if p is None:
+                return
+            self._link_summary = _part_to_frame(p, replace_parts=(('From Node', 'FromNode'), ('To Node', 'ToNode')))
         return self._link_summary
 
     @property
@@ -426,7 +396,6 @@ class SwmmReport:
         """
         if self._rainfall_file_summary is None:
             p = self._get_converted_part('Rainfall File Summary')
-            # p = '-'*10 + '\n' + p
             # p = p.replace('Data Source', 'DataSource')
             self._rainfall_file_summary = _part_to_frame(p)
         return self._rainfall_file_summary
@@ -441,9 +410,9 @@ class SwmmReport:
         """
         if self._raingage_summary is None:
             p = self._get_converted_part('Raingage Summary')
-            # p = '-'*10 + '\n' + p
-            p = p.replace('Data Source', 'DataSource')
-            self._raingage_summary = _part_to_frame(p)
+            if p is None:
+                return
+            self._raingage_summary = _part_to_frame(p, replace_parts=('Data Source', 'DataSource'))
         return self._raingage_summary
 
     @property
@@ -456,9 +425,9 @@ class SwmmReport:
         """
         if self._subcatchment_summary is None:
             p = self._get_converted_part('Subcatchment Summary')
-            # p = '-'*10 + '\n' + p
-            p = p.replace('Rain Gage', 'RainGage')
-            self._subcatchment_summary = _part_to_frame(p)
+            if p is None:
+                return
+            self._subcatchment_summary = _part_to_frame(p, replace_parts=('Rain Gage', 'RainGage'))
         return self._subcatchment_summary
 
     @property
@@ -471,7 +440,6 @@ class SwmmReport:
         """
         if self._cross_section_summary is None:
             p = self._get_converted_part('Cross Section Summary')
-            # p = '-'*10 + '\n' + p
             self._cross_section_summary = _part_to_frame(p)
         return self._cross_section_summary
 
@@ -524,9 +492,6 @@ class SwmmReport:
         """
         if self._node_surcharge_summary is None:
             p = self._get_converted_part('Node Surcharge Summary')
-            # if 'No nodes were surcharged.' in p:
-            #     self._node_surcharge_summary = pd.DataFrame()
-            # else:
             self._node_surcharge_summary = _part_to_frame(p)
         return self._node_surcharge_summary
 
@@ -540,9 +505,6 @@ class SwmmReport:
         """
         if self._node_flooding_summary is None:
             p = self._get_converted_part('Node Flooding Summary')
-            # if 'No nodes were flooded.' in p:
-            #     self._node_flooding_summary = pd.DataFrame()
-            # else:
             self._node_flooding_summary = _part_to_frame(p)
         return self._node_flooding_summary
 
@@ -556,11 +518,7 @@ class SwmmReport:
         """
         if self._storage_volume_summary is None:
             p = self._get_converted_part('Storage Volume Summary')
-
-            # for reading the table and accepting names shorten than 8 characters
-            p = p.replace('Storage Unit', 'Storage_Unit')
-
-            self._storage_volume_summary = _part_to_frame(p)
+            self._storage_volume_summary = _part_to_frame(p, replace_parts=('Storage Unit', 'Storage_Unit'))
         return self._storage_volume_summary
 
     @property
@@ -573,7 +531,7 @@ class SwmmReport:
         """
         if self._outfall_loading_summary is None:
             p = self._get_converted_part('Outfall Loading Summary')
-            self._outfall_loading_summary = _part_to_frame(p.replace('Outfall Node', 'Outfall_Node'))
+            self._outfall_loading_summary = _part_to_frame(p, replace_parts=('Outfall Node', 'Outfall_Node'))
         return self._outfall_loading_summary
 
     @property
@@ -600,8 +558,7 @@ class SwmmReport:
         if self._flow_classification_summary is None:
             p = self._get_converted_part('Flow Classification Summary')
             t = '---------- Fraction of Time in Flow Class ----------'
-            p = p.replace(t, ' ' * len(t))
-            self._flow_classification_summary = _part_to_frame(p)
+            self._flow_classification_summary = _part_to_frame(p, replace_parts=(t, ' ' * len(t)))
         return self._flow_classification_summary
 
     @property
@@ -614,15 +571,8 @@ class SwmmReport:
         """
         if self._conduit_surcharge_summary is None:
             p = self._get_converted_part('Conduit Surcharge Summary')
-
-            # --------------------------------------------
-            # if 'No conduits were surcharged.' in p:
-            #     self._conduit_surcharge_summary = pd.DataFrame()
-            #
-            # else:
-            p = p.replace('--------- Hours Full -------- ', 'HoursFull Hours Full HoursFull')
-            p = p.replace('Both Ends', 'Both_Ends')
-            self._conduit_surcharge_summary = _part_to_frame(p)
+            self._conduit_surcharge_summary = _part_to_frame(p, replace_parts=(('--------- Hours Full -------- ', 'HoursFull Hours Full HoursFull'),
+                                                                               ('Both Ends', 'Both_Ends')))
         return self._conduit_surcharge_summary
 
     @property
@@ -713,7 +663,9 @@ class SwmmReport:
             str: Node
         """
         if self._note is None:
-            self._note = ' '.join(self._get_converted_part('Note').strip(' *').split())
+            p = self._get_converted_part('Note')
+            if p:
+                self._note = ' '.join(p.strip(' *').split())
         return self._note
 
     @property
@@ -742,6 +694,19 @@ class SwmmReport:
             self._pumping_summary = _part_to_frame(p)
         return self._pumping_summary
 
+    @property
+    def street_flow_summary(self):
+        """
+        Street Flow Summary
+
+        Returns:
+            pandas.DataFrame: Street Flow Summary
+        """
+        if self._street_flow_summary is None:
+            p = self._get_converted_part('Street Flow Summary')
+            self._street_flow_summary = _part_to_frame(p, replace_parts=('Street Conduit', 'StreetConduit'))
+        return self._street_flow_summary
+
     @property
     def routing_time_step_summary(self):
         """
@@ -791,23 +756,41 @@ class SwmmReport:
         """
         if self._transect_summary is None:
             p = self._get_converted_part('Transect Summary')
-            self._transect_summary = {}
-            for transect in p.split('Transect')[1:]:
-                label, *data = transect.split()
-                self._transect_summary[label] = {}
-                sub = data[0][:-1]
-                d = []
-                for i in data[1:]:
-                    if i.endswith(':'):
-                        self._transect_summary[label][sub] = d
-                        sub = i[:-1]
-                    else:
-                        d.append(float(i))
-                self._transect_summary[label][sub] = d
-
-                self._transect_summary[label] = pd.DataFrame.from_dict(self._transect_summary[label])
+            self._transect_summary = _transect_street_shape_converter(p, key='Transect')
         return self._transect_summary
 
+    @property
+    def shape_summary(self):
+        """
+        Shape Summary
+
+        Returns:
+            dict[pandas.DataFrame]: Shape Summary
+        """
+        if self._shape_summary is None:
+            p = self._get_converted_part('Shape Summary')
+            self._shape_summary = _transect_street_shape_converter(p, key='Shape')
+        return self._shape_summary
+
+    @property
+    def street_summary(self):
+        """
+        Street Summary
+
+        Returns:
+            dict[pandas.DataFrame]: Street Summary
+        """
+        if self._street_summary is None:
+            p = self._get_converted_part('Street Summary')
+            self._street_summary = _transect_street_shape_converter(p, key='Street')
+        return self._street_summary
+        pass
+
+    # @property
+    # def most_frequent_nonconverging_nodes(self):
+    #     'Most Frequent Nonconverging Nodes'
+    #     return
+
     def get_simulation_info(self):
         """
         Simulation Infos
@@ -821,10 +804,13 @@ class SwmmReport:
 
     def _convert_str_time(self, s):
         import locale
-        orig_locale = locale.getlocale()
-        locale.setlocale(locale.LC_ALL, 'en_US.utf8')
-        date_time = datetime.datetime.strptime(s.strip(), '%a %b %d %H:%M:%S %Y')
-        locale.setlocale(locale.LC_ALL, orig_locale)
+        orig_locale = locale.getlocale(locale.LC_TIME)
+        try:
+            locale.setlocale(locale.LC_TIME, 'C')
+            date_time = datetime.datetime.strptime(s.strip(), '%a %b %d %H:%M:%S %Y')
+        except locale.Error:
+            date_time = s
+        locale.setlocale(locale.LC_TIME, orig_locale)
         return date_time
 
     @property
@@ -835,9 +821,9 @@ class SwmmReport:
         Returns:
             pd.Timestamp: Timestamp of the start of the simulation
         """
-        v = self.get_simulation_info()['Analysis begun on']
-        if v:
-            return self._convert_str_time(v)
+        info = self.get_simulation_info()
+        if info:
+            return self._convert_str_time(info['Analysis begun on'])
 
     @property
     def analyse_end(self):
@@ -847,9 +833,9 @@ class SwmmReport:
         Returns:
             pd.Timestamp: Timestamp of the end of the simulation
         """
-        v = self.get_simulation_info()['Analysis ended on']
-        if v:
-            return self._convert_str_time(v)
+        info = self.get_simulation_info()
+        if info:
+            return self._convert_str_time(info['Analysis ended on'])
 
     @property
     def analyse_duration(self):
@@ -859,16 +845,20 @@ class SwmmReport:
         Returns:
             pandas.Timedelta: simulation duration
         """
-        v = self.get_simulation_info()['Total elapsed time']
-        if v:
+        info = self.get_simulation_info()
+        if info:
+            v = info['Total elapsed time']
             if '< 1 sec' in v:
                 return datetime.timedelta(seconds=1)
+            di_td = {}
             if '.' in v:
-                return datetime.timedelta(days=v.split('.')[0]) + datetime.timedelta(*v.split('.')[1].split(':'))
-            if v.count(':') == 2:
-                return datetime.timedelta(**dict(zip(('hours', 'minutes', 'seconds'), (int(i) for i in v.strip().split(':')))))
+                days, v = v.split('.')
+                di_td['days'] = int(days)
 
-            return datetime.timedelta(*(int(i) for i in v.strip().split(':')))
+            if v.count(':') == 2:
+                di_td.update(dict(zip(('hours', 'minutes', 'seconds'), (int(i) for i in v.strip().split(':')))))
+
+            return datetime.timedelta(**di_td)
 
     @staticmethod
     def _pretty_dict(di, chunk_size=20):
@@ -906,17 +896,18 @@ class SwmmReport:
         Returns:
             dict[str, (str | bool | list[str])]: key is the error and value is a object label, a list of object-label or a bool
         """
-        t = self._raw_parts.get('Version+Title', None)
         di = {}
-        if t:
-            for line in t.split('\n'):
-                line = line.strip()
-                if line.startswith('ERROR'):
-                    label, txt = line.split(':', 1)
-                    if label in di:
-                        di[label].append(txt)
-                    else:
-                        di[label] = [txt]
+        for part in ('Version+Title', 'Analysis Options'):
+            t = self._raw_parts.get(part, None)
+            if t:
+                for line in t.split('\n'):
+                    line = line.strip()
+                    if line.startswith('ERROR'):
+                        label, txt = line.split(':', 1)
+                        if label in di:
+                            di[label].append(txt)
+                        else:
+                            di[label] = [txt]
         return di
 
     def print_errors(self):
