@@ -5,10 +5,12 @@ __email__ = "markus.pichler@tugraz.at"
 __version__ = "0.1"
 __license__ = "MIT"
 
-from ._run_helpers import get_report_errors, get_result_filenames, SWMMRunError
+from pathlib import Path
+
+from ._run_helpers import get_report_errors, get_result_filenames, SWMMRunError, TemporaryWorkingDirectory
 
 
-def swmm5_run_owa(fn_inp, fn_rpt=None, fn_out=None):
+def swmm5_run_owa(fn_inp, fn_rpt=None, fn_out=None, working_dir=None):
     """
     Run a simulation with OWA-SWMM (swmm-toolkit package).
 
@@ -18,6 +20,7 @@ def swmm5_run_owa(fn_inp, fn_rpt=None, fn_out=None):
         fn_inp (str | Path): pointer to name of input file (must exist)
         fn_rpt (str | Path): pointer to name of report file (to be created)
         fn_out (str | Path): pointer to name of binary output file (to be created)
+        working_dir (str or pathlib.Path): directory where swmm should be executed. Important if relative paths are in the input file. Default is directory of input-file.
     """
     fn_rpt_default, fn_out_default = get_result_filenames(fn_inp)
     if fn_rpt is None:
@@ -31,7 +34,11 @@ def swmm5_run_owa(fn_inp, fn_rpt=None, fn_out=None):
         raise (e, SWMMRunError('to run SWMM with OWA-SWMM you have to install the swmm-toolkit package (pip install swmm-toolkit).'))
 
     try:
-        solver.swmm_run(str(fn_inp), str(fn_rpt), str(fn_out))
+        if working_dir is None:
+            working_dir = Path(fn_inp).resolve().parent
+
+        with TemporaryWorkingDirectory(working_dir):
+            solver.swmm_run(str(fn_inp), str(fn_rpt), str(fn_out))
         print('', end='\r')  # solver doesn't write a last new-line
     except Exception as e:
         raise SWMMRunError(f'{e.args[0]}\n{fn_inp}\n{get_report_errors(fn_rpt)}')
